using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MATTR
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
      ClearWindows();
    }

    #region User interface actions

    private void AnalyzeFiles()
    {
      DialogResult r = openFileDialog1.ShowDialog();
      if (r != DialogResult.OK) return;

      Cursor.Current = Cursors.WaitCursor;

      List<string> files = new List<string>(openFileDialog1.FileNames);
      files.Sort();

      foreach (string s in files)
      {

        string ss = System.IO.File.ReadAllText(s);

        // Write filename into the details window
        textBoxDetails.AppendText(String.Format("\r\n\r\n{0}\r\n", System.IO.Path.GetFileName(s)));

        // Tokenize
        List<string> tt = TypeTokenTools.RemoveNonWords(TypeTokenTools.Tokenize(ss));

        // Compute MATTR
        int w = (int)numericUpDown1.Value;
        double m = TypeTokenTools.MATTR(tt,w);
        textBoxDetails.AppendText(String.Format("MATTR({0}): {1:0.000}\r\n",w,m));

        // Write in the results window
        textBoxResults.AppendText(String.Format("{1,7}\t{2,6}\t{3:0.000}  \t{0,-40}\r\n", System.IO.Path.GetFileName(s), tt.Count, w, m));

        // Write details, incl moving average values, in details window
        if (checkBoxShowAllWords.Checked)
        {
          Application.DoEvents();
          for(int i=0; i<tt.Count; i++)
            textBoxDetails.AppendText(String.Format("{2:######0}\t{1:0.000}\t{0}\r\n", tt[i], TypeTokenTools.WindowTTRs[i], i));
        }

        Cursor.Current = Cursors.Default;

      }

    }

    private void SaveResults()
    {
      saveFileDialog1.FileName = "MATTR-Results.txt";

      DialogResult r = saveFileDialog1.ShowDialog();
      if (r != DialogResult.OK) return;

      System.IO.File.WriteAllText(saveFileDialog1.FileName, textBoxResults.Text, Encoding.ASCII);
    }

    private void SaveDetails()
    {
      saveFileDialog1.FileName = "MATTR-Details.txt";

      DialogResult r = saveFileDialog1.ShowDialog();
      if (r != DialogResult.OK) return;

      System.IO.File.WriteAllText(saveFileDialog1.FileName, textBoxDetails.Text, Encoding.ASCII);
    }


    private string VersionInfo()
    {
      System.Version v = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version;

      // v.Build is days since Jan. 1, 2000
      // v.Revision*2 is seconds since local midnight
      // (NEVER daylight saving time)

      DateTime t = new DateTime(v.Build * TimeSpan.TicksPerDay +
                                  v.Revision * TimeSpan.TicksPerSecond * 2
                                ).AddYears(1999);

      return Application.ProductName + " " + v.ToString() + " " + t.ToString("yyyy/MM/dd");
    }

    private void ClearWindows()
    {
      textBoxDetails.Clear();
      textBoxDetails.Text = VersionInfo() + "\r\n";

      textBoxResults.Clear();
      textBoxResults.Text = VersionInfo() + "\r\n";
      textBoxResults.AppendText(
        String.Format("\r\n{0,7}\t{1,6}\t{2,6}\t{3}\r\n","Words","Window","MATTR ","Filename"));
    }


    #endregion

    #region Menu choices and buttons

    private void buttonAnalyzeFiles_Click(object sender, EventArgs e)
    {
      AnalyzeFiles();
    }

    private void buttonSaveResults_Click(object sender, EventArgs e)
    {
      SaveResults();
    }

    private void buttonSaveDetails_Click(object sender, EventArgs e)
    {
      SaveDetails();    
    }

    #endregion

    private void analyzeFilesToolStripMenuItem_Click(object sender, EventArgs e)
    {
      AnalyzeFiles();
    }

    private void saveResultsToolStripMenuItem_Click(object sender, EventArgs e)
    {
      SaveResults();
    }

    private void saveDetailsToolStripMenuItem_Click(object sender, EventArgs e)
    {
      SaveDetails();
    }

    private void exitToolStripMenuItem_Click(object sender, EventArgs e)
    {
      Application.Exit();
    }

    private void clearOutputWindowsToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ClearWindows();
    }

    private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
    {
      AboutBox1 ab = new AboutBox1();
      ab.ShowDialog();
    }
    

  }
}